@@ -11,63 +11,62 @@ import (
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/client_golang/prometheus/promauto"
 	"github.com/urfave/cli/v3"
-	"gitlab.com/gitlab-org/gitaly/v16"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/backup"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/bootstrap"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/bootstrap/starter"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/bundleuri"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/cache"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/cgroups"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git/catfile"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git/gitcmd"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git/housekeeping"
-	housekeepingmgr "gitlab.com/gitlab-org/gitaly/v16/internal/git/housekeeping/manager"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git/localrepo"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/config"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/config/sentry"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/hook"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/hook/updateref"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/maintenance"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/server"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/service"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/service/setup"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/counter"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/keyvalue"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/keyvalue/databasemgr"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/mdfile"
-	nodeimpl "gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/node"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/raftmgr"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/storagemgr"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/storagemgr/partition"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/storagemgr/partition/migration"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage/storagemgr/partition/migration/reftable"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/transaction"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitlab"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/grpc/backchannel"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/grpc/client"
-	housekeepingmw "gitlab.com/gitlab-org/gitaly/v16/internal/grpc/middleware/housekeeping"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/grpc/middleware/limithandler"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/grpc/protoregistry"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/helper"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/helper/env"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/limiter"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/limiter/watchers"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/log"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/offloading"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/streamcache"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/tempdir"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/tracing"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/version"
+	"gitlab.com/gitlab-org/gitaly/v18"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/backup"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/bootstrap"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/bootstrap/starter"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/bundleuri"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/cache"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/cgroups"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git/catfile"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git/gitcmd"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git/housekeeping"
+	housekeepingmgr "gitlab.com/gitlab-org/gitaly/v18/internal/git/housekeeping/manager"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git/localrepo"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/config"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/config/sentry"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/hook"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/hook/updateref"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/maintenance"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/server"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/service"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/service/setup"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/counter"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/keyvalue"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/keyvalue/databasemgr"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/mdfile"
+	nodeimpl "gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/node"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/raftmgr"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/storagemgr"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/storagemgr/partition"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/storagemgr/partition/migration"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage/storagemgr/partition/migration/reftable"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/transaction"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitlab"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/grpc/backchannel"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/grpc/client"
+	housekeepingmw "gitlab.com/gitlab-org/gitaly/v18/internal/grpc/middleware/housekeeping"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/grpc/middleware/limithandler"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/grpc/protoregistry"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/helper"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/helper/env"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/limiter"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/limiter/watchers"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/log"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/offloading"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/streamcache"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/tempdir"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/tracing"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/version"
 	"gitlab.com/gitlab-org/labkit/fips"
 	"gitlab.com/gitlab-org/labkit/monitoring"
-	labkittracing "gitlab.com/gitlab-org/labkit/tracing"
 	"go.uber.org/automaxprocs/maxprocs"
 	"gocloud.dev/blob"
 	"google.golang.org/grpc"
 
 	// Import to register the proxy codec with gRPC.
-	_ "gitlab.com/gitlab-org/gitaly/v16/internal/grpc/proxy"
+	_ "gitlab.com/gitlab-org/gitaly/v18/internal/grpc/proxy"
 )
 
 func newServeCommand() *cli.Command {
@@ -111,6 +110,12 @@ func serveAction(ctx context.Context, cmd *cli.Command) error {
 		return cli.Exit(err, 1)
 	}
 
+	_, tracingCloser, err := tracing.InitializeTracerProvider(context.Background(), "gitaly")
+	if err != nil {
+		logger.WithError(err).Error("error initializing distributed tracing")
+	}
+	defer func() { _ = tracingCloser.Close() }()
+
 	if cfg.Auth.Transitioning && len(cfg.Auth.Token) > 0 {
 		logger.Warn("Authentication is enabled but not enforced because transitioning=true. Gitaly will accept unauthenticated requests.")
 	}
@@ -163,7 +168,6 @@ func configure(configPath string) (config.Cfg, log.Logger, error) {
 
 	sentry.ConfigureSentry(logger, version.GetVersion(), sentry.Config(cfg.Logging.Sentry))
 	cfg.Prometheus.Configure(logger)
-	labkittracing.Initialize(labkittracing.WithServiceName("gitaly"))
 	preloadLicenseDatabase(logger)
 
 	return cfg, logger, nil
@@ -189,7 +193,7 @@ func run(appCtx *cli.Command, cfg config.Cfg, logger log.Logger) error {
 	beganRun := time.Now()
 
 	bootstrapSpan, ctx := tracing.StartSpan(ctx, "gitaly-bootstrap", nil)
-	defer bootstrapSpan.Finish()
+	defer bootstrapSpan.End()
 
 	if cfg.RuntimeDir != "" {
 		if err := config.PruneOldGitalyProcessDirectories(logger, cfg.RuntimeDir); err != nil {
@@ -277,6 +281,7 @@ func run(appCtx *cli.Command, cfg config.Cfg, logger log.Logger) error {
 			},
 		)),
 		client.WithDialOptions(
+			grpc.WithStatsHandler(tracing.NewGRPCClientStatsHandler()),
 			client.UnaryInterceptor(),
 			client.StreamInterceptor(),
 		),
@@ -461,6 +466,23 @@ func run(appCtx *cli.Command, cfg config.Cfg, logger log.Logger) error {
 			return fmt.Errorf("new node manager: %w", err)
 		}
 		defer nodeMgr.Close()
+		node = nodeMgr
+
+		assignmentWorkerErrCh := make(chan error, 1)
+		go func() {
+			defer close(assignmentWorkerErrCh)
+			started := time.Now()
+			err := storagemgr.AssignmentWorker(ctx, cfg, nodeMgr, dbMgr, locator)
+			duration := time.Since(started)
+
+			if err != nil {
+				logger.WithField("duration_ms", duration.Milliseconds()).WithError(err).Error("partition assignment worker failed")
+			} else {
+				logger.WithField("duration_ms", duration.Milliseconds()).Info("partition assignment worker completed")
+			}
+
+			assignmentWorkerErrCh <- err
+		}()
 
 		if cfg.Raft.Enabled {
 			for _, storageCfg := range cfg.Storages {
@@ -473,20 +495,10 @@ func run(appCtx *cli.Command, cfg config.Cfg, logger log.Logger) error {
 				}
 			}
 			node = raftNode
+		}
 
-			// Start partition worker synchronously as it is a pre-requisite for Raft.
-			if err := storagemgr.AssignmentWorker(ctx, cfg, node, dbMgr, locator); err != nil {
-				return fmt.Errorf("partition assignment worker: %w", err)
-			}
-
-			if err := replicaPartitionMigration(cfg.Storages, locator, dbMgr, logger, true); err != nil {
-				return err
-			}
-		} else {
-			if err := replicaPartitionMigration(cfg.Storages, locator, dbMgr, logger, false); err != nil {
-				return err
-			}
-			node = nodeMgr
+		if err := replicaPartitionMigration(cfg, locator, dbMgr, logger, assignmentWorkerErrCh); err != nil {
+			return err
 		}
 
 		reftableMigrator := reftable.NewMigrator(logger, reftableMigratorMetrics, node, localrepoFactory)
@@ -766,7 +778,7 @@ func run(appCtx *cli.Command, cfg config.Cfg, logger log.Logger) error {
 	if err := b.Start(); err != nil {
 		return fmt.Errorf("unable to start the bootstrap: %w", err)
 	}
-	bootstrapSpan.Finish()
+	bootstrapSpan.End()
 	// There are a few goroutines running async tasks that may still be in progress (i.e. preloading the license
 	// database), but this is a close enough indication of startup latency.
 	logger.WithField("duration_ms", time.Since(beganRun).Milliseconds()).Info("Started Gitaly")
@@ -795,8 +807,8 @@ func run(appCtx *cli.Command, cfg config.Cfg, logger log.Logger) error {
 }
 
 // replicaPartitionMigration performs replica partition migrations for all storages.
-func replicaPartitionMigration(storages []config.Storage, locator storage.Locator, dbMgr *databasemgr.DBManager, logger log.Logger, raftEnabled bool) error {
-	for _, storageCfg := range storages {
+func replicaPartitionMigration(cfg config.Cfg, locator storage.Locator, dbMgr *databasemgr.DBManager, logger log.Logger, assignmentWorkerErrCh <-chan error) error {
+	for _, storageCfg := range cfg.Storages {
 		partitionsDir, err := locator.PartitionsDir(storageCfg.Name)
 		if err != nil {
 			return fmt.Errorf("retrieving partitions dir for storage %s: %w", storageCfg.Name, err)
@@ -821,7 +833,7 @@ func replicaPartitionMigration(storages []config.Storage, locator storage.Locato
 		var migrate func() error
 		var actionDesc, completionDesc string
 
-		if raftEnabled {
+		if cfg.Raft.Enabled {
 			// migrate partitions to be raft replica compatible
 			shouldMigrate = !migrated
 			migrate = migrator.Forward
@@ -836,11 +848,16 @@ func replicaPartitionMigration(storages []config.Storage, locator storage.Locato
 		}
 
 		if shouldMigrate {
+			// Block and wait for assignment worker to complete before migration
+			if err := <-assignmentWorkerErrCh; err != nil {
+				return fmt.Errorf("partition assignment worker: %w", err)
+			}
+
 			logger.Info(fmt.Sprintf("%s for storage: %s", actionDesc, storageCfg.Name))
 			startTime := time.Now()
 
 			if err := migrate(); err != nil {
-				if raftEnabled {
+				if cfg.Raft.Enabled {
 					return fmt.Errorf("migrating replica partitions: %w", err)
 				}
 				return fmt.Errorf("undoing replica partitions migration: %w", err)
